module materials
    ! this module actually plays as both geometry and material module, materials are defined directly on geometries.
    ! set detailed geometry information here for material module to operate
! Geometry ver.1 
! Pin to Plane geometry based on Kulikovsky test case
! it is in cylindrical axis, with x=r, y=z
! (y/0.01)^2-(x/0.0018)^2=1 : Pin shape
! in other words, if 1+(x/0.0018)^2>(y/0.01)^2, then it is the metal pin other wise plasma
    use Data_structure
    use Constants
    use materials_lib
    use meshes
    
    contains
    
    subroutine SET_material()! this subroutine has to be userdefined
    ! if 1+(x/0.018)^2>(y/0.01)^2, then it is the metal pin other wise plasma
    integer i,j,k,p
    integer debug
    double precision x,y,f(4),g(4)
    debug=0
    ! below to calculate 4 corner points positions and compare with pin geometry arc
    do i=1,index_max
        x=cell(i)%boundary_cell(4)%X
        y=cell(i)%boundary_cell(1)%Y
        f(1)=y-1.d4*x**2-1.6d-2
        g(1)=y-2*x-1.6d-2
        x=cell(i)%boundary_cell(2)%X
        y=cell(i)%boundary_cell(1)%Y    
        f(2)=y-1.d4*x**2-1.6d-2
        g(2)=y-2*x-1.6d-2
        x=cell(i)%boundary_cell(2)%X
        y=cell(i)%boundary_cell(3)%Y    
        f(3)=y-1.d4*x**2-1.6d-2
        g(3)=y-2*x-1.6d-2
        x=cell(i)%boundary_cell(4)%X
        y=cell(i)%boundary_cell(3)%Y    
        f(4)=y-1.d4*x**2-1.6d-2
        g(4)=y-2*x-1.6d-2
        
        !******************************original settings for Tholin's pin-pin dishcrages************************************************!
        ! 1: set all domain as neutral gas, this material is just used to reduce plasma region to have more speed
 ! 1: set all domain as neutral gas, this material is just used to reduce plasma region to have more speed
        cell(i)%material=>Library_material(5)
        do j=1,4
        cell(i)%boundary_cell(j)%material=>Library_material(5)
        end do
        
        
        ! 2: cover plasma
         if ((cell(i)%boundary_cell(2)%X<=3.1d-2).and.(cell(i)%boundary_cell(4)%X>=8.d-3).and.(cell(i)%boundary_cell(3)%Y<=0.21d-2)) then
            cell(i)%material=>Library_material(1)
            do j=1,4
                cell(i)%boundary_cell(j)%material=>Library_material(1)
            end do
         end if
         
        !!!! 3: cover dielectric over plasma
        if ((cell(i)%Y<5.d-4).and.(cell(i)%X<=3.d-2)) then
            cell(i)%material=>Library_material(4)
            do j=1,4
                cell(i)%boundary_cell(j)%material=>Library_material(4)
            end do
        end if
        
        ! 4: cover driven electrode over plasma
        do j=1,4
            if ((cell(i)%Y<=5.5d-4).and.(cell(i)%Y>=5.d-4).and.(cell(i)%boundary_cell(2)%X<=1.d-2)) then    
            ! above generates a rectangular electrode
            !if ((cell(i)%Y<=8.d-4).and.(cell(i)%Y>=5.d-4).and.(0.0003*cell(i)%X + 0.00025*cell(i)%Y <= 0.000003175)) then    
                cell(i)%material=>Library_material(3)
                do k=1,4
                    cell(i)%boundary_cell(k)%material=>Library_material(3)
                end do
            end if
        end do
        
        
        ! 5: cover ground electrode over plasma, this is only assigned to the boundary of a inner cell.
        !if ((cell(i)%boundary_cell(1)%Y.eq.0).AND.(cell(i)%boundary_cell(2)%X<=1.99d-2).AND.(cell(i)%X>0.5d-2)) then
        ! here there is a very often mistake: Poisson equation will give unphysical results if dielectric boundary is given a newmann boundary condition, as written in comment above.
        if ((cell(i)%boundary_cell(1)%Y.eq.0)) then
                cell(i)%boundary_cell(1)%material=>Library_material(2)
        end if
    end do

    
	call SET_fixmaterial()
	
    end subroutine SET_material    
    
    
end module materials